
.display-arrangement {
  background-color: #8b8b8b;
}

.display-arrangement.monitor {
  border: solid 1px @borders;
  margin: 0px 0px 1px 1px;
  background: @theme_bg_color;
  padding: 0.4em;
}

.display-arrangement.monitor.primary {
  border-bottom: 0.4em solid black;
}

.display-arrangement.monitor:selected {
  border: solid .2em black;
}

.display-arrangement.monitor.primary:selected {
  border-top: solid .2em black;
  border-left: solid .2em black;
  border-right: solid .2em black;
  border-bottom: 0.4em solid black;
}

.display-arrangement.monitor-label {
  font-size: larger;
  font-weight: bold;
  border-radius: 0.3em;
  padding-right: 0.2em;
  padding-left: 0.2em;
  color: #fff;
  background: #000;
}

